// Sprocket Framework header file
// Window.h


#ifndef		_WINDOW_
#define		_WINDOW_

#ifndef		__TYPES__
#include	<Types.h>
#endif

#ifndef		__DRAG__
#include	<Drag.h>
#endif

#ifndef		_MENUBAR_
#include	"MenuBar.h"
#endif


typedef	short	WindowTemplateID;


class	TWindow
	{
protected:
	//	protect the constructor because it isnt legal to create a TWindow directly
	
						TWindow();
public:

	virtual				~TWindow();

	//	Event routing methods

	virtual	Boolean		EventFilter(EventRecord * theEvent);	

	//	Methods you shouldnt need to override, but might need to

	enum WindowType
		{
		kNormalWindow = 0,
		kFloatingWindow,
		kModalWindow
		};

	virtual void		CreateWindow(WindowType typeOfWindowToCreate = kNormalWindow);
	virtual	Boolean		Select(void);
	virtual	void		Drag(Point startPoint);
	virtual void		Nudge(short horizontalDistance, short verticalDistance);
	virtual void		Grow(Point startPoint);
	virtual void		Zoom(short zoomState);

	virtual	void		ShowHide(Boolean showFlag);


	//	Methods which MUST be overridden:

	virtual WindowRef	MakeNewWindow(WindowRef behindWindow)	= 0;
	

	//	Methods which probably should be overridden

	virtual	void		AdjustCursor(EventRecord * anEvent);
	virtual void		Idle(EventRecord * anEvent);
	virtual void		Activate(Boolean activating);
	virtual void		Draw(void);
	virtual void		Click(EventRecord * anEvent);
	virtual	void		KeyDown(EventRecord * anEvent);

	virtual void		GetPerfectWindowSize(Rect * perfectSize);
	virtual	void		GetWindowSizeLimits(Rect * limits);
	virtual void		AdjustForNewWindowSize(Rect * oldRect,Rect * newRect);


	//	Window property accessor methods
	//		watch for new ones when we add scripting support	
	
	virtual	Boolean		IsVisible(void);
	
	virtual	Boolean		CanClose(Boolean quitting);		
	virtual	Boolean		Close(void);
	virtual	Boolean		DeleteAfterClose(void);
	
	//	Methods for handling menus & menu commands
	
	virtual	void		AdjustMenusBeforeMenuSelection(void);
	virtual Boolean		DoMenuSelection(short menu, short item);
	virtual Boolean		DoCommand(CommandID menuCommand);

	//	Methods for use with the Drag Manager
	
	virtual	void		ClickAndDrag(EventRecord * anEvent);
						//	NOTE:	This method takes GLOBAL coordinates
						//			because thats what TrackDrag expects.
	
	virtual	OSErr		HandleDrag(DragTrackingMessage dragMessage,DragReference theDrag);

	virtual	OSErr		DragEnterWindow(DragReference theDrag);
	virtual	OSErr		DragInWindow(DragReference theDrag);
	virtual	OSErr		DragLeaveWindow(DragReference theDrag);

	virtual	OSErr		HandleDrop(DragReference theDragRef);

	virtual Boolean		IsPointInContentRgn( Point pt );
	virtual Boolean		IsMouseInContentRgn( DragReference dragRef );
	virtual Rect		GetContentsBounds(void);

protected:
	WindowRef			fWindow;
	WindowType			fWindowType;

	Boolean				fIsDialogWindow;
	Boolean				fIsVisible;

public:	
	static RgnHandle	fgScratchRgn;
	static short		fgNumWindows;
	};


//	Utility Functions:

//	Dont you just wish you didnt have to do this?
pascal WindowRef	GetNewColorOrBlackAndWhiteWindow(short windowID, void *wStorage, WindowRef behind);
pascal WindowRef	NewColorOrBlackAndWhiteWindow(void *wStorage, const Rect *boundsRect, ConstStr255Param title, Boolean visible, short theProc, WindowRef behind, Boolean goAwayFlag, long refCon);

void				DrawJustTheGrowIcon(WindowRef theWindow);

TWindow *			GetWindowObject(WindowRef aWindow);

WindowRef			FrontModalWindow(void);
WindowRef			LastModalWindow(void);
WindowRef			FrontFloatingWindow(void);
WindowRef			LastFloatingWindow(void);
WindowRef			FrontNonFloatingWindow(void);

void				HiliteAndActivateWindow(WindowRef aWindow,Boolean active);
void				SuspendResumeWindows(Boolean resuming);
void				HiliteWindowsForModalDialog(Boolean hiliting);

pascal OSErr		CallWindowDragTrackingHandler(DragTrackingMessage message,WindowRef theWindow,void *handlerRefCon,DragReference theDragRef);
pascal OSErr		CallWindowDragReceiveHandler(WindowRef theWindow, void *handlerRefCon,DragReference theDragRef);

#endif
